Type: Package
Package: logger
Authors@R: c(
    person("Gergely", "Daróczi", , "daroczig@rapporter.net", role = c("aut", "cre"), comment = c(ORCID = "0000-0003-3149-8537")),
    person("System1", role = c("fnd"))
    )
Title: A Lightweight, Modern and Flexible Logging Utility
Description: Inspired by the the 'futile.logger' R package and 'logging' Python module, this utility provides a flexible and extensible way of formatting and delivering log messages with low overhead.
Version: 0.3.0
Date: 2024-03-03
URL: https://daroczig.github.io/logger/
BugReports: https://github.com/daroczig/logger/issues
Encoding: UTF-8
RoxygenNote: 7.3.1
License: AGPL-3
Imports: utils
Suggests: glue, pander, jsonlite, crayon, slackr (>= 1.4.1),
        RPushbullet, telegram, testthat, covr, knitr, rmarkdown,
        devtools, roxygen2, parallel, rsyslog, shiny, callr, txtq,
        botor, R.utils, syslognet
Enhances: logging, futile.logger, log4r
VignetteBuilder: knitr
Config/testthat/edition: 2
NeedsCompilation: no
Packaged: 2024-03-05 14:10:49 UTC; daroczig
Author: Gergely Daróczi [aut, cre] (<https://orcid.org/0000-0003-3149-8537>),
  System1 [fnd]
Maintainer: Gergely Daróczi <daroczig@rapporter.net>
Repository: CRAN
Date/Publication: 2024-03-05 16:20:02 UTC
Built: R 4.4.0; ; 2024-04-29 18:01:33 UTC; unix
