% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tester.R
\name{logger_tester_function}
\alias{logger_tester_function}
\title{Testing logging from package}
\usage{
logger_tester_function(level, msg)
}
\arguments{
\item{level}{foo}

\item{msg}{bar}
}
\description{
Testing logging from package
}
